@@ -9,7 +9,7 @@
 */
 
 /* ====================================================================
- * Copyright (c) 1998-2001 Ralf S. Engelschall. All rights reserved.
+ * Copyright (c) 1998-2004 Ralf S. Engelschall. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -253,7 +253,8 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 ap_ctx_set(ap_global_ctx, "ssl::handshake::timeout", (void *)FALSE);
                 return;
             }
-            else if (ERR_GET_REASON(ERR_peek_error()) == SSL_R_HTTP_REQUEST) {
+            else if ((ERR_GET_REASON(ERR_peek_error()) == SSL_R_HTTP_REQUEST) &&
+                     (ERR_GET_LIB(ERR_peek_error()) == ERR_LIB_SSL)) {
                 /*
                  * The case where OpenSSL has recognized a HTTP request:
                  * This means the client speaks plain HTTP on our HTTPS
@@ -333,6 +334,12 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 ap_ctx_set(ap_global_ctx, "ssl::handshake::timeout", (void *)FALSE);
                 return;
             }
+            else if (   (SSL_get_error(ssl, rc) == SSL_ERROR_WANT_READ  && BIO_should_retry(SSL_get_rbio(ssl)))
+                     || (SSL_get_error(ssl, rc) == SSL_ERROR_WANT_WRITE && BIO_should_retry(SSL_get_wbio(ssl)))) {
+                ssl_log(srvr, SSL_LOG_TRACE, "SSL handshake I/O retry (server %s, client %s)",
+                        cpVHostID, conn->remote_ip != NULL ? conn->remote_ip : "unknown");
+                continue;
+            }
             else {
                 /*
                  * Ok, anything else is a fatal error
@@ -964,11 +971,11 @@ int ssl_hook_Access(request_rec *r)
             certstack = SSL_get_peer_cert_chain(ssl);
             cert = SSL_get_peer_certificate(ssl);
             if (certstack == NULL && cert != NULL) {
-                /* client cert is in the session cache, but there is
-                   no chain, since ssl3_get_client_certificate()
-                   sk_X509_shift'ed the peer cert out of the chain.
-                   So we put it back here for the purpose of quick
-                   renegotiation.  */
+                /* client certificate is in the SSL session cache, but
+                   there is no chain, since ssl3_get_client_certificate()
+                   sk_X509_shift()'ed the peer certificate out of the
+                   chain. So we put it back here for the purpose of quick
+                   renegotiation. */
                 certstack = sk_new_null();
                 sk_X509_push(certstack, cert);
             }
@@ -995,10 +1002,12 @@ int ssl_hook_Access(request_rec *r)
             SSL_set_verify_result(ssl, certstorectx.error);
             X509_STORE_CTX_cleanup(&certstorectx);
             if (SSL_get_peer_cert_chain(ssl) != certstack) {
-                /* created by us, so free it */
+                /* created by us above, so free it */
                 sk_X509_pop_free(certstack, X509_free);
             }
-            X509_free(cert);
+            else {
+                /* X509_free(cert); not necessary AFAIK --rse */
+            }
         }
         else {
             /* do a full renegotiation */
@@ -1136,7 +1145,6 @@ int ssl_hook_Auth(request_rec *r)
 {
     SSLSrvConfigRec *sc = mySrvConfig(r->server);
     SSLDirConfigRec *dc = myDirConfig(r);
-    char b1[MAX_STRING_LEN], b2[MAX_STRING_LEN];
     char *clientdn;
     const char *cpAL;
     const char *cpUN;
@@ -1197,12 +1205,11 @@ int ssl_hook_Auth(request_rec *r)
      * adding the string "xxj31ZMTZzkVA" as the password in the user file.
      * This is just the crypted variant of the word "password" ;-)
      */
-    ap_snprintf(b1, sizeof(b1), "%s:password", clientdn);
-    ssl_util_uuencode(b2, b1, FALSE);
-    ap_snprintf(b1, sizeof(b1), "Basic %s", b2);
-    ap_table_set(r->headers_in, "Authorization", b1);
+    cpAL = ap_pstrcat(r->pool, "Basic ", ap_pbase64encode(r->pool,
+        ap_pstrcat(r->pool, clientdn, ":password", NULL)), NULL);
+    ap_table_set(r->headers_in, "Authorization", cpAL);
     ssl_log(r->server, SSL_LOG_INFO,
-            "Faking HTTP Basic Auth header: \"Authorization: %s\"", b1);
+            "Faking HTTP Basic Auth header: \"Authorization: %s\"", cpAL);
 
     return DECLINED;
 }
