@@ -9,7 +9,7 @@
 */
 
 /* ====================================================================
- * Copyright (c) 1998-1999 Ralf S. Engelschall. All rights reserved.
+ * Copyright (c) 1998-2004 Ralf S. Engelschall. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -239,7 +239,7 @@ static char *ssl_expr_eval_func_file(request_rec *r, char *filename)
         *buf = NUL;
     }
     else {
-        if ((buf = (char *)ap_palloc(r->pool, sizeof(char) * len+1)) == NULL) {
+        if ((buf = (char *)ap_palloc(r->pool, sizeof(char) * (len+1))) == NULL) {
             ssl_expr_error = "Cannot allocate memory";
             ap_pfclose(r->pool, fp);
             return "";
